@@ -5,7 +5,7 @@
 void text_print_text(GtkText * widget, char *fmt, ...);
 void port_dialog(char *message);
 void download_dialog(char *filename, int mime);
-void changes_dialog(char *title);
+void changes_dialog();
 void server_dialog(char *title);
 GtkWidget *create_info_win(char *text, char **buttons,
 			   GtkSignalFunc * func);
@@ -14,3 +14,4 @@ void delete_dialog(char *text);
 void resume_delete_dialog(char *text);
 void new_version_dialog(char *text);
 void check_version_dialog(char *text);
+void info_dialog(char *text);
