@@ -84,17 +84,6 @@
 #define getpid() GetCurrentProcessId()
 #endif
 
-#ifndef _
-/* This is for other GNU distributions with internationalized messages.
-   When compiling libc, the _ macro is predefined.  */
-#ifdef HAVE_LIBINTL_H
-#include <libintl.h>
-#define _(msgid)	gettext (msgid)
-#else
-#define _(msgid)	(msgid)
-#endif
-#endif
-
 /* This version of `getopt' appears to the caller like standard Unix `getopt'
    but it behaves differently for the user, since it allows the user
    to intersperse the options with the other arguments.
@@ -152,7 +141,7 @@ static char *nextchar;
 /* Callers store zero here to inhibit the error message
    for unrecognized options.  */
 
-int opterr = 1;
+int opterr = 0;
 
 /* Set to an option character which was unrecognized.
    This must be initialized on some systems to avoid linking in the
@@ -603,7 +592,7 @@ int long_only;
 
     if (ambig && !exact) {
       if (opterr)
-	fprintf(stderr, _("%s: option `%s' is ambiguous\n"),
+	fprintf(stderr, "%s: option `%s' is ambiguous\n",
 		argv[0], argv[optind]);
       nextchar += strlen(nextchar);
       optind++;
@@ -624,12 +613,12 @@ int long_only;
 	    if (argv[optind - 1][1] == '-')
 	      /* --option */
 	      fprintf(stderr,
-		      _("%s: option `--%s' doesn't allow an argument\n"),
+		      "%s: option `--%s' doesn't allow an argument\n",
 		      argv[0], pfound->name);
 	    else
 	      /* +option or -option */
 	      fprintf(stderr,
-		      _("%s: option `%c%s' doesn't allow an argument\n"),
+		      "%s: option `%c%s' doesn't allow an argument\n",
 		      argv[0], argv[optind - 1][0], pfound->name);
 
 	  nextchar += strlen(nextchar);
@@ -643,7 +632,7 @@ int long_only;
 	else {
 	  if (opterr)
 	    fprintf(stderr,
-		    _("%s: option `%s' requires an argument\n"),
+		    "%s: option `%s' requires an argument\n",
 		    argv[0], argv[optind - 1]);
 	  nextchar += strlen(nextchar);
 	  optopt = pfound->val;
@@ -669,11 +658,11 @@ int long_only;
       if (opterr) {
 	if (argv[optind][1] == '-')
 	  /* --option */
-	  fprintf(stderr, _("%s: unrecognized option `--%s'\n"),
+	  fprintf(stderr, "%s: unrecognized option `--%s'\n",
 		  argv[0], nextchar);
 	else
 	  /* +option or -option */
-	  fprintf(stderr, _("%s: unrecognized option `%c%s'\n"),
+	  fprintf(stderr, "%s: unrecognized option `%c%s'\n",
 		  argv[0], argv[optind][0], nextchar);
       }
       nextchar = (char *) "";
@@ -697,9 +686,9 @@ int long_only;
       if (opterr) {
 	if (posixly_correct)
 	  /* 1003.2 specifies the format of this message.  */
-	  fprintf(stderr, _("%s: illegal option -- %c\n"), argv[0], c);
+	  fprintf(stderr, "%s: illegal option -- %c\n", argv[0], c);
 	else
-	  fprintf(stderr, _("%s: invalid option -- %c\n"), argv[0], c);
+	  fprintf(stderr, "%s: invalid option -- %c\n", argv[0], c);
       }
       optopt = c;
       return '?';
@@ -723,7 +712,7 @@ int long_only;
       } else if (optind == argc) {
 	if (opterr) {
 	  /* 1003.2 specifies the format of this message.  */
-	  fprintf(stderr, _("%s: option requires an argument -- %c\n"),
+	  fprintf(stderr, "%s: option requires an argument -- %c\n",
 		  argv[0], c);
 	}
 	optopt = c;
@@ -764,7 +753,7 @@ int long_only;
 	}
       if (ambig && !exact) {
 	if (opterr)
-	  fprintf(stderr, _("%s: option `-W %s' is ambiguous\n"),
+	  fprintf(stderr, "%s: option `-W %s' is ambiguous\n",
 		  argv[0], argv[optind]);
 	nextchar += strlen(nextchar);
 	optind++;
@@ -779,8 +768,8 @@ int long_only;
 	    optarg = nameend + 1;
 	  else {
 	    if (opterr)
-	      fprintf(stderr, _("\
-%s: option `-W %s' doesn't allow an argument\n"), argv[0], pfound->name);
+	      fprintf(stderr, "%s: option `-W %s' doesn't allow an argument\n",
+		      argv[0], pfound->name);
 
 	    nextchar += strlen(nextchar);
 	    return '?';
@@ -791,7 +780,7 @@ int long_only;
 	  else {
 	    if (opterr)
 	      fprintf(stderr,
-		      _("%s: option `%s' requires an argument\n"),
+		      "%s: option `%s' requires an argument\n",
 		      argv[0], argv[optind - 1]);
 	    nextchar += strlen(nextchar);
 	    return optstring[0] == ':' ? ':' : '?';
@@ -829,7 +818,7 @@ int long_only;
 	  if (opterr) {
 	    /* 1003.2 specifies the format of this message.  */
 	    fprintf(stderr,
-		    _("%s: option requires an argument -- %c\n"),
+		    "%s: option requires an argument -- %c\n",
 		    argv[0], c);
 	  }
 	  optopt = c;
