@@ -2,89 +2,93 @@
    This is free software distributed under the terms of the
    GNU Public License.  See the file COPYING for details. */
 
+#ifndef GLOBAL_H_
+#define GLOBAL_H_
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include "gtk/gtk.h"
+
+#include "interface.h"
+#include "lopster.h"
+
+#ifdef HAVE_ZLIB
+#  ifdef THREADS_ENABLED
+#    define VERSION_STRING "Lopster [zt] "VERSION
+#  else
+#    define VERSION_STRING "Lopster [z] "VERSION
+#  endif
+#else
+#  ifdef THREADS_ENABLED
+#    define VERSION_STRING "Lopster [t] "VERSION
+#  else
+#    define VERSION_STRING "Lopster "VERSION
+#  endif
+#endif
+
 extern const char LineSpeedShort[12][20];
 extern char tstr[10][512];
 extern char *list[10];
-extern char Network[5][20];
+extern char Network[N_NUMBER][20];
 extern char *Browser[BROWSER_NO];
 extern int SocketStatus[S_NUMBER];
-extern const GtkTargetEntry lib_target_table[1];
+extern char* NetType[5];
+extern const int ModeMask[N_NUMBER];
+extern const char* ModeNames[MODE_NUMBER];
+extern const char* CModeNames[CMODE_NUMBER];
+
+#define LIB_LIST_TARGETS   2
+extern const GtkTargetEntry lib_list_targets[LIB_LIST_TARGETS];
 
 extern char **environ;
+extern GtkCTree* DownloadTree[4];
+extern GtkWidget* DownloadScroll[4];
+extern GtkWidget* DownloadVbox[4];
 
-#ifdef WORDS_BIGENDIAN
-#define BSWAP16(x) ((((x) & 0xff) << 8) | (((x) >> 8) & 0xff))
-#define BSWAP32(x) (((x) << 24) | (((x) & 0x0000ff00) << 8) | ((x) >> 24) | (((x) & 0x00ff0000) >> 8))
-#else
-#define BSWAP16(x) (x)
-#define BSWAP32(x) (x)
+#define DL_GET_NICK(download) \
+  (((download) && (download)->nets)?((net_user_t*)((download)->nets->data))->user:NULL)
+#define DL_GET_NET(download) \
+  (((download) && (download)->nets)?((net_user_t*)((download)->nets->data))->net:NULL)
+
+#define NET_IRC(net) ((net) && ((net)->type & NETWORK_IRC))
+#define NET_NAP(net) ((net) && ((net)->type & NETWORK_NAPSTER))
+#define NET_ONLINE(net) ((net) && (net)->active_server)
+#define NET_CONNECTED(net) ((net) && (net)->active_server && (net)->active_server->status == SERVER_ONLINE)
+
+#ifndef MSG_NOSIGNAL
+#define MSG_NOSIGNAL 0
 #endif
 
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
-# define ATTR_UNUSED __attribute__((unused))
-# define ATTR_NORETURN __attribute__((noreturn))
-# define ATTR_PRINTFLIKE(fmt,var) __attribute__((format(printf,fmt,var)))
-#else
-# define ATTR_UNUSED
-# define ATTR_NORETURN
-# define ATTR_PRINTFLIKE(fmt,var)
-#endif
-
-#define l_strcasecmp g_strcasecmp
-#define l_malloc g_malloc
-#define l_free g_free
-#define l_strdup g_strdup
-#define l_strdup_printf g_strdup_printf
-
 extern global_t global;
 
 int ColorTable(int i1, int i2);
 int ColorTable2(int index);
 
-/* scan and format functions */
-unsigned long extract_bytes(char* string);
-int extract_duration(char* string);
-char *print_bytes(char *str, unsigned long bytes);
-char* print_duration(char* str, int seconds);
-char *print_size(char *str, double bytes);
-char *print_time_short(char *str, int secs);
-char *print_time(char *str, int secs);
-char *print_time_average(char *str, int secs);
-char *print_speed(char *str, long bytes, int withdot);
-
 /* */
 void detect_line_pixs(int speed, GdkPixmap ** pixmap, GdkBitmap ** bitmap);
-void detect_speed_pixs(double rate, GdkPixmap ** pixmap, GdkBitmap ** bitmap);
+void down_speed_pixs(double rate, GdkPixmap ** pixmap, GdkBitmap ** bitmap);
+void up_speed_pixs(double rate, GdkPixmap ** pixmap, GdkBitmap ** bitmap);
 
-/* path/filename functions */
-char *extract_last_dir(char *filename);
-char *extract_filename(char *filename);
-void convert_local_name(char *fname);
-char *rename_file(char *filename);
-char *extract_short_name(char *longname);
-char *convert_to_win(char *name);
-char *convert_to_unix(char *name);
+/* string to int functions */
+int speed2int     (char *linespeed);
+int level2int     (char *level);
+int mime2int      (char *mime);
+int status2int    (char *status);
+int timestamp2int (char *stamp);
+int browser2int   (char *browser);
 
-/* string to int_variable functions */
-int speed2int(char *linespeed);
-int level2int(char *level);
-int mime2int(char *mime);
-int mime2int_(char *mime);
-int status2int(char *status);
-int timestamp2int(char* stamp);
-int browser2int(char *browser);
-int channelmode2int(char* mode);
-
-/* int variable to string functions */
-char *LineSpeed(int no);
-char *MimeNames(int no);
-char *MimeNames_(int no);
-char *UserStatus(int status);
-char *TimeStamp(int id);
-char *WebBrowser(int browser);
-char* ChannelMode(int id);
-char* Level(int level);
-char* LevelShort(int level);
+/* int to string functions */
+char* LineSpeed   (int no);
+char* TimeNames   (int no);
+char* MimeNames   (int no, int mode);
+char* MimeNames_  (int no, int mode);
+char* UserStatus  (int status);
+char* TimeStamp   (int id);
+char* WebBrowser  (int browser);
+char* Level       (int level);
+char* LevelShort  (int level);
 
 void browse_url(char *url);
 int version_is_up_to_date(char *version);
@@ -94,37 +98,26 @@ int guess_mime(char *suffix);
 int find_valid_bitrate(int bitrate);
 char *current_time(char *stime, int with_secs);
 char *ntoa(unsigned int ip);
-int directory_exists(char *dir);
-void create_dir(char *dir);
 
-void update_status_line(int highlight);
+void update_reply(char *user, net_t *net, int highlight);
+void update_status_line();
+#ifndef HAVE_STRCASESTR
 char *strcasestr(char *str, char *sub);
-
+#endif
 
 
 int idle_function(gpointer data);
-void read_geometry();
-void write_geometry();
 
-void read_rc();
-void write_rc();
-
-void setup_sensitive(int section);
-void setup_commands();
-void create_pixmaps();
-void setup_main_tab();
-void setup_widgets();
-void setup_buttons();
-void setup_mime();
+void set_title();
 void setup_global_vars();
+void setup_main_tab();
+void write_rc();
 
 void global_init();
 void global_exit();
 
-void destroy_file_row(gpointer data);
-
-
-
+gint browse_saved_compare(GtkCList * clist,
+			  gconstpointer ptr1, gconstpointer ptr2);
 gint channel_compare(GtkCList * clist,
 		     gconstpointer ptr1, gconstpointer ptr2);
 gint socket_compare(GtkCList * clist,
@@ -137,6 +130,8 @@ gint download_compare2(GtkCList * clist,
 		       gconstpointer ptr1, gconstpointer ptr2);
 gint browse_compare(GtkCList * clist,
 		    gconstpointer ptr1, gconstpointer ptr2);
+gint lib_compare(GtkCList * clist,
+		 gconstpointer ptr1, gconstpointer ptr2);
 gint hotlist_compare(GtkCList * clist,
 		     gconstpointer ptr1, gconstpointer ptr2);
 gint access_compare(GtkCList * clist,
@@ -147,11 +142,6 @@ gint resume_compare(GtkCList * clist,
 		    gconstpointer ptr1, gconstpointer ptr2);
 
 void send_search_request();
-void cmd_eval_whois(char *data);
-void cmd_eval_whowas(char *data);
-void cmd_whowas(char *data);
-
-void cmd_server_stats(char *data);
 
 void make_list_from_string(GList ** glist, char *text, char *sep);
 char *make_string_from_list(GList * glist, char *sep);
@@ -168,37 +158,26 @@ int toggle_button_state(GtkWidget * button);
 void set_up_button(GtkWidget * button, char *pix1, char *pix2, char *wid);
 void set_up_button2(GtkWidget * button,
 		    GtkWidget * pix1, GtkWidget * pix2, GtkWidget * wid);
-void show_bans(char *channel);
-void cmd_banlist_entry(char *data);
+void show_bans(net_t* net, char *channel);
 char *cparse(char *input);
-void client_message(char *text, const char *fmt, ...);
-void server_message(char *text, const char *fmt, ...);
-void protocol_message(int direction, int type, char* text);
-void cmd_sping(char *data);
-void cmd_ping(char *data);
-void cmd_pong(char *data);
-
-void cmd_set_port(char *data);
+void client_message(const char *text, const char *fmt, ...);
+void server_message(net_t* net, const char *fmt, ...);
+void protocol_message(net_t* net, int direction, int type, char* text);
 
 void toggle_friend(char *user);
 void toggle_enemy(char *user);
 void toggle_nodownload(char *user);
 
 void opennap_notify();
-int opennap_version(int major, int minor);
-char *arg(char *data, int last);
-void convert_chars(char *text, char from, char to);
 
-GtkWidget *create_customize_win(GtkCList * clist);
-void show_clients();
-void show_global_users(char *args, int clear);
-void convert_file(char *name);
+GtkWidget *create_customize_win(GtkCList * clist, int* array);
+void show_clients(net_t* net);
+void show_global_users(net_t* net, char *args);
 
-file_t *file_dup(file_t * file);
-int get_mimetype(char *name);
+int get_media_type(char *name);
 suffix_t *get_suffix(char *name);
 GtkWidget *create_mode_popup(void);
-GtkWidget *create_user_popup(int mode);
+GtkWidget *create_user_popup(int mode, void* data);
 void draw_band_width(bandwidth_t * bandwidth, int full);
 
 void extra_win_get_geometry();
@@ -206,14 +185,14 @@ void advance_progress(char *message, gfloat per);
 GtkWidget *main_tab_detach(int no);
 
 clone_t *is_ip_in_clones(GList * glist, char *ip);
-int get_version(char *data);
-void get_server(char *data);
+int get_version(net_t* net, const char *data);
+void get_server(net_t* net, char *data);
 void clones_clear();
 
 GList *create_allowed_ports(char *string);
 int get_next_port(GList * list, int last);
 char *create_string_from_ports(GList * list);
-void destroy_allowed_ports(GList * list);
+void destroy_string_list(GList * list);
 int port_is_allowed(GList * list, int port);
 
 void gtk_my_widget_show(GtkWidget * widget);
@@ -224,5 +203,21 @@ int notebook_page_visible(int no);
 int notebook_page_current();
 
 void print_topright_corner();
-void set_user_level(int level);
+void set_user_level(net_t* net, int level);
 void file_move(char* src, char* dest);
+void ban_user(net_t* net, char* user, char* reason, long timeout);
+void update_queue_length();
+void send_user_status(net_t* net, char* user);
+
+char *resolveip(const char *ip);
+void ban_networks_load();
+int ban_networks_scan(const char *ip);
+
+void download_layout(download_layout_t* new_layout);
+
+double global_queue_size();
+int chmode2int_lop(char* mode, int* chmode1, int* chmode2);
+char* int2chmode_lop(int set, int clear);
+
+
+#endif
