@@ -4,12 +4,12 @@
 
 GtkWidget *create_hotlist_popup();
 
-void hotlist_add(char *nick, char *des, int visible);
+hot_t* hotlist_add(char *nick, char *des, int visible);
 void hotlist_remove(char *nick, int visible);
 hot_t *hotlist_search_user(char *user);
 
 void hotlist_update_user(user_t * userinfo);
 void hotlist_show_description(hot_t* hot);
 void hotlist_user_unknown(hot_t * hot);
-void hotlist_user_offline(hot_t * hot);
-void hotlist_user_online(hot_t * hot);
+void hotlist_user_online(net_t* net, hot_t * hot);
+void hotlist_user_offline(net_t* net, hot_t * hot);
