@@ -6,24 +6,21 @@
 #  include <config.h>
 #endif
 
-#include <string.h>
-#include <strings.h>
-#include <stdlib.h>
+#ifdef HAVE_LIBLCONV_H
+#  include <liblconv.h>
+#endif
+
 #include <getopt.h>
-#include <unistd.h>
-#include <sys/time.h>
-#include <time.h>
 #include <stdio.h>
-#include <netdb.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
 
-#include <gtk/gtk.h>
-
-#include "lopster.h"
 #include "support.h"
 #include "global.h"
+#include "exec.h"
 #include "splash.h"
-#include "interface.h"
-#include "chat.h"
+#include "utils.h"
 
 struct option long_options[] = {
   {"help", 0, 0, 0},
@@ -40,21 +37,20 @@ int splash;
 void display_usage(void)
 {
   fprintf(stderr,
-	  _("\nUsage: lopster [options]\n\n"
-	    "Options:\n"
-	    "--------\n\n"
-	    "-h, --help                   Display this text and exit.\n"
-	    "-d, --dir                    Use other config folder than (default: $HOME/.lopster)\n"
-	    "-c, --config                 Use other rc-file than (default: lopsterrc in config folder)\n"
-	    "                             Use with CAUTION!!\n"
-	    "-a, --away                   Start Lopster in afk mode\n"
-	    "-n, --no-splash              Start Lopster without splash screen\n"
-	    "-v, --version                Print version number and exit.\n\n"));
+	  "\nUsage: lopster [options]\n\n"
+	  "Options:\n"
+	  "--------\n\n"
+	  "-h, --help                   Display this text and exit.\n"
+	  "-d, --dir                    Use other config folder than (default: $HOME/.lopster)\n"
+	  "-c, --config                 Use other rc-file than (default: lopsterrc in config folder)\n"
+	  "                             Use with CAUTION!!\n"
+	  "-a, --away                   Start Lopster in afk mode\n"
+	  "-n, --no-splash              Start Lopster without splash screen\n"
+	  "-v, --version                Print version number and exit.\n\n");
   exit(0);
 }
 
-void parse_cmd_line(int argc, char **argv)
-{
+void parse_cmd_line(int argc, char **argv) {
   gint c, o;
 
   while ((c = getopt_long(argc, argv, "hd:c:a:nv", long_options, &o)) != -1) {
@@ -66,15 +62,15 @@ void parse_cmd_line(int argc, char **argv)
 	break;
       case 1:
 	if (optarg)
-	  global.options.config_dir = strdup(optarg);
+	  global.options.config_dir = l_strdup(optarg);
 	break;
       case 2:
 	if (optarg)
-	  global.options.config_file = strdup(optarg);
+	  global.options.config_file = l_strdup(optarg);
 	break;
       case 3:
 	if (optarg) {
-	  global.afk.message = strdup(optarg);
+	  global.afk.message = l_strdup(optarg);
 	  global.afk.since = time(NULL);
 	}
 	break;
@@ -82,7 +78,10 @@ void parse_cmd_line(int argc, char **argv)
 	splash = 0;
 	break;
       case 5:
-	printf("Lopster %s\n", VERSION);
+	if (INTERN_MICRO_VERSION)
+	  printf(VERSION_STRING".%d\n", INTERN_MICRO_VERSION);
+	else
+	  printf(VERSION_STRING"\n");
 	exit(0);
 	break;
       }
@@ -92,15 +91,15 @@ void parse_cmd_line(int argc, char **argv)
       break;
     case 'd':
       if (optarg)
-	global.options.config_dir = strdup(optarg);
+	global.options.config_dir = l_strdup(optarg);
       break;
     case 'c':
       if (optarg)
-	global.options.config_file = strdup(optarg);
+	global.options.config_file = l_strdup(optarg);
       break;
     case 'a':
       if (optarg) {
-	global.afk.message = strdup(optarg);
+	global.afk.message = l_strdup(optarg);
 	global.afk.since = time(NULL);
       }
       break;
@@ -108,7 +107,10 @@ void parse_cmd_line(int argc, char **argv)
       splash = 0;
       break;
     case 'v':
-      printf("Lopster %s\n", VERSION);
+      if (INTERN_MICRO_VERSION)
+	printf(VERSION_STRING".%d\n", INTERN_MICRO_VERSION);
+      else
+	printf(VERSION_STRING"\n");
       exit(0);
       break;
     }
@@ -125,9 +127,15 @@ int main(int argc, char *argv[])
 
   splash = 1;
 
+#ifdef HAVE_LIBLCONV_H
+  lconv_set_locale();
+#endif
+  srandom(time(NULL));
   gtk_set_locale();
   gtk_init(&argc, &argv);
 
+  exec_init();
+
   add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps");
   add_pixmap_directory(PACKAGE_SOURCE_DIR "/pixmaps");
 
@@ -137,7 +145,7 @@ int main(int argc, char *argv[])
   if (splash) global.splash_win = splash_screen();
   else global.splash_win = NULL;
 
-  advance_progress(_("Starting Lopster"), 0.05);
+  advance_progress("Starting Lopster", 0.05);
   global.win = create_window();
 
   // dont really show it, only add it to queue
@@ -146,14 +154,18 @@ int main(int argc, char *argv[])
 
   while (gtk_events_pending())
     gtk_main_iteration();
+  // test
+  while (gdk_events_pending())
+    gdk_flush();
+
   //  gdk_window_raise (global.splash_win->window);
   //  gtk_window_set_transient_for (GTK_WINDOW(global.splash_win), GTK_WINDOW(global.win));
 
-  advance_progress(_("Initializing..."), 0.10);
+  advance_progress("Initializing...", 0.10);
   global_init();
-  advance_progress(_("Initialization done."), 1);
+  advance_progress("Initialization done.", 1);
 
-  if (global.afk.message) set_afk(global.afk.message);
+  //  if (global.afk.message) set_afk(global.afk.message);
 
   if (global.splash_win) {
     gtk_widget_destroy(global.splash_win);
