@@ -35,10 +35,8 @@
 #define P_TAB_STYLE    0x04000000
 #define P_SCHEMES       0x08000000	// page schemes
 #define P_ADVANCED      0x10000000	// page advanced
-#define P_ALIASES       0x20000000	// page aliases
-#define P_CHANNELS      0xc0000000	// page channels
-#define P_CHANNEL      0x40000000	// page channels
-#define P_REJOIN       0x80000000	// page channels
+#define P_NETWORKS      0x20000000	
+#define P_REJOIN       0x40000000	// page channels
 
 #define P_NONE          0x00000000
 #define P_ALL           0xfffffffe
@@ -51,3 +49,5 @@ void clear_shared_folder();
 void clear_suffixes();
 GtkWidget *create_pipe_popup(int val);
 GtkWidget *create_pipe2_popup(int val);
+
+void on_dir_select343(gchar * dir);
