@@ -17,11 +17,11 @@ void enemy_add(char *item);
 void enemy_remove(char *item);
 void ignore_add(char *item);
 void ignore_remove(char *item);
-void opchannel_add(char *item);
-void opchannel_remove(char *item);
-void autojoin_add(char *item);
-void autojoin_remove(char *item);
+void banreason_add(char *item);
+void banreason_remove(char *item);
 void nodownload_add(char *item);
 void nodownload_remove(char *item);
+void afkreason_add(char *item);
+void afkreason_remove(char *item);
 
 GtkWidget *create_string_list_popup(int slist);
